<?php
/**
 * @filename ReaderInterface.php
 * @author Jerry Yan <792602257@qq.com>
 * @date 2020/12/17 14:58
 */


namespace JerryYan\DSL\Reader;


/**
 * 解析器需要使用到的读取类
 * @package JerryYan\DSL\Reader
 * @author Jerry Yan <792602257@qq.com>
 * @date 2020/12/17 15:37
 */
abstract class ReaderInterface
{
    protected $currentLine = 1;
    protected $currentPosition = 0;
    protected $currentLinePosition = 0;
    protected $nextPosition = 0;

    /**
     * 获取下一个字符
     * @return string
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/18 12:06
     */
    abstract public function getNextChar(): string;

    /**
     * 获取当前识别符
     * @return string
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/18 12:06
     */
    abstract public function getCurrentToken(): string;

    /**
     * 获取下一个识别符
     * @return string
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/18 12:06
     */
    abstract public function getNextToken(): string;

    /**
     * 移动至下一个识别符
     * @return bool
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/18 12:06
     */
    abstract public function moveToNextToken(): bool;

    /**
     * 跳过当前行
     * @return bool
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/17 15:43
     */
    abstract public function skipCurrentLine(): bool;

    /**
     * 从当前位置跳到结束位置
     * @param string $end
     * @return bool
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/17 15:43
     */
    abstract public function skipUntil(string $end="*/"): bool;

    /**
     * 重置读取器
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/18 12:01
     */
    public function reset(): void
    {
        $this->currentLine = 1;
        $this->currentPosition = 0;
        $this->nextPosition = 0;
        $this->moveToNextToken();
    }

    /**
     * 重置读取器
     * @author Jerry Yan <792602257@qq.com>
     * @date 2020/12/18 12:01
     */
    public function resetCursor(): void
    {
        $this->reset();
    }
}